import datetime
import os
import psutil
from django.template.defaultfilters import slugify
from django.conf import settings

def get_module_list():
    default_modules = (
        'bigbrother.core.UserCount',
        'bigbrother.core.NewUsersTodayCount',
        'bigbrother.core.FreeRamCount',
        'bigbrother.core.FreeDiskCount',
    )
    return getattr(settings, 'BIGBROTHER_MODULES', default_modules)

class BigBrotherModule():
    name = 'Unamed Module'
    write_to_db = True
    prepend_text = ''
    add_text = ''
    warning_low = None
    warning_high = None
    
    def get_val(self):
        return None
        
    def get_text(self):
        return '%s%g%s' % (self.prepend_text, self.get_val(), self.add_text)
        
    def get_slug(self):
        return slugify(self.name)

    def warning(self):
        if self.get_val() >= self.warning_high and self.warning_high != None:
            return True
        if self.get_val() <= self.warning_low and self.warning_low != None:
            return True
        return False

class UserCount(BigBrotherModule):
    name = 'Total Users'
    
    def get_val(self):
        from django.contrib.auth.models import User
        users = User.objects.all()
        return users.count()

        
class NewUsersTodayCount(BigBrotherModule):
    name = 'New Users Today'
    
    def get_val(self):
        from django.contrib.auth.models import User
        curday = datetime.datetime.today()
        users = User.objects.filter(date_joined__year=curday.year, date_joined__month=curday.month, date_joined__day=curday.day)
        return users.count()


class FreeRamCount(BigBrotherModule):
    name = 'Free RAM'
    add_text = ' MB'
    warning_low = 16

    def get_val(self):
        return psutil.phymem_usage()[2] / (1024 * 1024)

class SwapUsage(BigBrotherModule):
    name = 'Swap Usage'
    add_text = ' MB'
    warning_high = 1

    def get_val(self):
        return psutil.virtmem_usage()[1] / (1024 * 1024)

class FreeDiskCount(BigBrotherModule):
    name = 'Free Disk Space'
    add_text = ' GB'
    
    def get_val(self):
        s = os.statvfs('/')
        return round((s.f_bavail * s.f_frsize) / ( 1024 * 1024 * 1024.0 ), 1)



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding model 'ModuleStat'
        db.create_table('bigbrother_modulestat', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('modulename', self.gf('django.db.models.fields.CharField')(max_length=64)),
            ('added', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('value', self.gf('django.db.models.fields.IntegerField')()),
        ))
        db.send_create_signal('bigbrother', ['ModuleStat'])


    def backwards(self, orm):
        
        # Deleting model 'ModuleStat'
        db.delete_table('bigbrother_modulestat')


    models = {
        'bigbrother.modulestat': {
            'Meta': {'object_name': 'ModuleStat'},
            'added': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modulename': ('django.db.models.fields.CharField', [], {'max_length': '64'}),
            'value': ('django.db.models.fields.IntegerField', [], {})
        }
    }

    complete_apps = ['bigbrother']



from django.db import models

class ModuleStat(models.Model):
    modulename = models.CharField(max_length=64)
    added = models.DateTimeField(auto_now_add=True)
    value = models.DecimalField(max_digits=10, decimal_places=2)


from django.conf import settings
from django.utils.importlib import import_module
from django.http import HttpResponse
from django.shortcuts import render_to_response
from django.template import RequestContext

from bigbrother.models import ModuleStat
from bigbrother.core import get_module_list


def index(request):
    
    bb = []

    for m in get_module_list():
        modulename, attr = m.rsplit('.', 1)
        module = import_module(modulename)
        name = getattr(module,attr)().name
        text = getattr(module,attr)().get_text()
        value = getattr(module,attr)().get_val()
        warning = getattr(module,attr)().warning()
        bb.append({'name':name, 'value':value, 'text':text, 'warning':warning})
    
    return render_to_response('bigbrother/index.html', {'bb': bb}, context_instance=RequestContext(request)) 

def graph(request, slug):
    q = ModuleStat.objects.filter(modulename=slug)
    if q.count() >= 7: week = q[q.count()-7:]
    else: week = q
    if q.count() >= 31: month = q[q.count()-31:]
    else: month = q
    if q.count() >= 365: year = q[q.count()-365:]
    else: year = q
    
    if q:
        lastdow = week[week.count()-1].added
        lastdom = month[month.count()-1].added
        lastdoy = year[year.count()-1].added
    return render_to_response('bigbrother/graph.html', locals(), context_instance=RequestContext(request))

def update(request):
    
    bb = []
    
    for m in get_module_list():
        modulename, attr = m.rsplit('.', 1)
        module = import_module(modulename)
        
        if getattr(module,attr)().write_to_db:
            name = getattr(module,attr)().get_slug()
            value = getattr(module,attr)().get_val()
            bb.append({'name':name, 'value':value})
            
    for mo in bb:
        ModuleStat(modulename=mo['name'], value=mo['value']).save()
    
    return HttpResponse(bb)
            

    
